#ifndef PITCL_PACK_H
#define PITCL_PACK_H
/* -*- c -*-
 *
 * pack.h --
 *
 *	Global declarations of piTcl, for the packer management.
 *
 * Copyright (C) 1998 Andreas Kupries (a.kupries@westend.com)
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL I BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS
 * SOFTWARE AND ITS DOCUMENTATION, EVEN IF I HAVE BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * I SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND
 * I HAVE NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 *
 * CVS: $Id: pack.h,v 1.2 1999/06/28 21:20:36 aku Exp $
 */

#ifdef __cplusplus
extern "C" {
#endif

#include <tcl.h>


/* The following structure describes the (un)packer commands/scripts
 * to use while accessing a database. All references can be NULL,
 * indicating a transfer of unchanged binary data.
 *
 * -*- PORTING NOTE -*-
 * The results of binary transfers are unreliable for tcl versions
 * below 8.0
 *
 * Naming conventions used:
 * - Prefixes 'p' (pack) and 'u' (unpack).
 *
 * All commands/scripts are called with the data to convert appended,
 * the result of the evaluation is interpreted as result of the
 * conversion too. No special global variables or some such.
 */

typedef struct PiTclPackInfo {

  char* name;            /* Name of the converter described by the structure */
  int   refCount;        /* Number of references to this packer structure. The
			  * structure can be freed the moment this value drops
			  * below 1. */
  Tcl_HashEntry* hPtr;   /* Backreference to the entry in the global registry. */
  Tcl_Obj* pRecordCmd;   /* Pack the record/resource information into binary */
  Tcl_Obj* uRecordCmd;   /* Unpack binary record/resource information */
  Tcl_Obj* pAppInfoCmd;  /* Pack the application information into binary */
  Tcl_Obj* uAppInfoCmd;  /* Unpack binary application information */
  Tcl_Obj* pSortInfoCmd; /* Pack the sort information into binary */
  Tcl_Obj* uSortInfoCmd; /* Unpack binary sort information */

} PiTclPackInfo;


/* Global management structure, contains all registered packers.
 */

typedef struct PiTclPackers {
  Tcl_HashTable cvt;      /* Table of all registered packers */
} PiTclPackers;


/* Management of packer structures
 */

int
PiTclPackInit _ANSI_ARGS_ ((PiTclPackers* p));

int
PiTclPackFree _ANSI_ARGS_ ((PiTclPackers* p));


PiTclPackInfo*
PiTclPackCreate _ANSI_ARGS_ ((char* name));

int
PiTclPackRelease _ANSI_ARGS_ ((PiTclPackInfo* p));

void
PiTclPackPreserve _ANSI_ARGS_ ((PiTclPackInfo* p));

int
PiTclPackConfigure _ANSI_ARGS_ ((PiTclPackInfo* p,
				 Tcl_Interp*    interp,
				 char*          cmd,
				 int            objc,
				 Tcl_Obj* CONST objv []));

Tcl_Obj*
PiTclPackCGet _ANSI_ARGS_ ((PiTclPackInfo* p,
			    Tcl_Interp*    interp,
			    char*          cmd,
			    Tcl_Obj*       option));

int
PiTclPackFind _ANSI_ARGS_ ((PiTclPackers*   p,
			    Tcl_Interp*     interp,
			    Tcl_Obj*        name,
			    PiTclPackInfo** res));

int
PiTclPackList _ANSI_ARGS_ ((Tcl_Interp*    interp,
			    PiTclPackers*  p));

int
PiTclPackAdd _ANSI_ARGS_ ((PiTclPackers*  p,
			   Tcl_Interp*    interp,
			   PiTclPackInfo* pa));

int
PiTclPackRemove _ANSI_ARGS_ ((PiTclPackers*  p,
			      Tcl_Interp*    interp,
			      PiTclPackInfo* pa));

#ifdef __cplusplus
}
#endif /* C++ */
#endif /* PITCL_PACK_H */
