#ifndef PITCL_WRAP_H
#define PITCL_WRAP_H
/* -*- c -*-
 *
 * wrap.h --
 *
 *	Global declarations of piTcl for wrapper definition and usage
 *
 * Copyright (C) 1998 Andreas Kupries (a.kupries@westend.com)
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL I BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS
 * SOFTWARE AND ITS DOCUMENTATION, EVEN IF I HAVE BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * I SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND
 * I HAVE NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 *
 * CVS: $Id: wrap.h,v 1.2 1999/06/28 21:20:36 aku Exp $
 */

#ifdef __cplusplus
extern "C" {
#endif

#include <tclUtil.h>
#include <tcl.h>


/*
 * Definitions for conversion of a bytestream into a tcl list and vice
 * versa. A 2-step process is defined: conversion from the bytestream
 * to a C structure, then conversion of the structure to a tcl list
 * using the utilities defined in 'tclUtil'.
 */

/* Definition of the procedure interfaces used for
 * bytestream <-> struct conversion.
 */

typedef int PiTcl_CvtStruct  _ANSI_ARGS_ ((void* s, char* str, int len));
typedef int PiTcl_FreeStruct _ANSI_ARGS_ ((void* s));

/* Definition of conversion descriptor structure.
 */

typedef struct PiTcl_CvtData {
  TclUtil_Struct*   structDef;    /* Definition for conversion between the
				   * intermediate structure and a tcl list */
  int               structSize;   /* Size of the intermediate structure in
				   * bytes */
  PiTcl_FreeStruct* structFree;   /* Free all memory allocated and refered from
				   * the inside of the structure. NULL is
				   * allowed and indicates a structure without
				   * internal pointers. */
  PiTcl_CvtStruct*  structPack;   /* Pack the structure into a bytestream */
  PiTcl_CvtStruct*  structUnpack; /* Unpack a bytestream into a structure */
  int               streamSize;   /* Usual size of a bytestream to pack a
				   * structure into, in bytes. Initially
				   * allocated. Extended as required. */
} PiTcl_CvtData;


/* Overall descriptor structure specific to our task of wrapping pilot
 * database packers. Embraces converters for 'records'/'resources', 'app-'
 * and 'sortinfo. The latter two can be NULL, indicating a database without
 * such information.
 */

typedef struct PiTcl_PilotCvt {
 PiTcl_CvtData* record;   /* Conversion descriptor for records/resources */
 PiTcl_CvtData* sortinfo; /* Conversion descriptor for sort information */
 PiTcl_CvtData* appinfo;  /* Conversion descriptor for application information
			   */
} PiTcl_PilotCvt;


/* Generic wrapper.
 */

int
PiTclPilotCvtCmd _ANSI_ARGS_ ((ClientData     cd, /* <= PiTcl_PilotCvt */
			       Tcl_Interp*    interp,
			       int            objc,
			       Tcl_Obj* CONST objv []));

#ifdef __cplusplus
}
#endif /* C++ */
#endif /* PITCL_WRAP_H */
